require 'nn'

local ViterbiCriterion, parent = torch.class('nn.ViterbiCriterion','nn.Criterion')

function ViterbiCriterion:__init(nc)
   parent.__init(self)
   self.nClass = nc
   self.normalize = false
end


function ViterbiCriterion:forward(input,target)
   local nerror = 0
   for i=1,input:size(1) do
      --print(input[i] .. ' .. ' .. target[i])
      if target[i] ~= input[i] then
	 nerror = nerror + 1
      end
   end
   self.output = nerror

   if self.normalize then
--      print('[f] normalize by ' .. input:size(1))
--      self.output = self.output / input:size(1)
   end

   return self.output
end


-- for each sample in the batch of samples
-- insert 1 for the wrong class, and -1 for the correct one
-- each column in the output is supposed to contain a pair of 1,-1
function ViterbiCriterion:backward(input, target)
   self.gradInput:resize(self.nClass,input:size(1))
   self.gradInput:zero()
   for i=1,input:size(1) do
      if target[i] ~= input[i] then
	 self.gradInput[input[i]][i] = 1
	 self.gradInput[target[i]][i] = -1
      end
   end

   if self.normalize then
--      print('[b] normalize by ' .. input:size(1))
      self.gradInput:div(input:size(1))
   end

   return self.gradInput
end
